@@ -4,9 +4,9 @@ use strict;
 use vars qw/$VERSION @ISA @EXPORT_OK %EXPORT_TAGS/;
 use Exporter;
 @ISA = 'Exporter';
-@EXPORT_OK = qw/capture capture_exec qxx/;
+@EXPORT_OK = qw/capture capture_exec qxx capture_exec_combined qxy/;
 %EXPORT_TAGS = (all => \@EXPORT_OK);
-$VERSION = '1.04_01';
+$VERSION = '1.04_02';
 
 sub capture (&@) { ## no critic
     my ($code, $output, $error) = @_;
@@ -15,19 +15,30 @@ sub capture (&@) { ## no critic
         $$_ = '' unless defined($$_);
     }
     my $capture_out = IO::CaptureOutput::_proxy->new('STDOUT', $output);
-    my $capture_err = IO::CaptureOutput::_proxy->new('STDERR', $error);
+    my $capture_err = IO::CaptureOutput::_proxy->new(
+        'STDERR', $error, $output == $error ? 'STDOUT' : undef
+    );
     &$code();
 }
 
 sub capture_exec {
     my @args = @_;
     my ($output, $error);
-    capture sub { system _shell_quote(@args)}, \$output, \$error;
+    capture sub { system _shell_quote(@args) }, \$output, \$error;
     return wantarray ? ($output, $error) : $output;
 }
 
 *qxx = \&capture_exec;
 
+sub capture_exec_combined {
+    my @args = @_;
+    my $output;
+    capture sub { system _shell_quote(@args) }, \$output, \$output;
+    return $output;
+}
+
+*qxy = \&capture_exec_combined;
+
 # extra quoting required on Win32 systems
 *_shell_quote = ($^O =~ /MSWin32/) ? \&_shell_quote_win32 : sub {@_};
 sub _shell_quote_win32 {
@@ -47,26 +58,44 @@ sub _shell_quote_win32 {
 # and then transfers it to a scalar reference
 package IO::CaptureOutput::_proxy;
 use File::Temp 'tempfile';
+use File::Basename qw/basename/;
 use Symbol qw/gensym qualify qualify_to_ref/;
 use Carp;
 
+sub _is_wperl { $^O eq 'MSWin32' && basename($^X) eq 'wperl.exe' }
+
 sub new {
     my $class = shift;
-    my ($fh, $capture) = @_;
+    my ($fh, $capture, $merge_fh) = @_;
     $fh       = qualify($fh);         # e.g. main::STDOUT
     my $fhref = qualify_to_ref($fh);  # e.g. \*STDOUT
 
     # Duplicate the filehandle
-    my $saved = gensym;
-    open $saved, ">&$fh" or croak "Can't redirect <$fh> - $!";
+    my $saved;
+    {
+        no strict 'refs'; # for 5.005
+        if ( defined fileno($fh) && ! _is_wperl() ) {
+            $saved = gensym;
+            open $saved, ">&$fh" or croak "Can't redirect <$fh> - $!";
+        }
+    }
 
-    # Create replacement filehandle
-    my $newio = gensym;
-    (undef, my $newio_file) = tempfile;
-    open $newio, "+>$newio_file" or croak "Can't create temp file for $fh - $!";
+    # Create replacement filehandle if not merging
+    my ($newio, $newio_file);
+    if ( ! $merge_fh ) {
+        $newio = gensym;
+        (undef, $newio_file) = tempfile;
+        open $newio, "+>$newio_file" or croak "Can't create temp file for $fh - $!";
+    }
+    else {
+        $newio = qualify($merge_fh);
+    }
 
-    # Redirect
-    open $fhref, ">&".fileno($newio) or croak "Can't redirect $fh - $!";
+    # Redirect (or merge)
+    {
+        no strict 'refs'; # for 5.005
+        open $fhref, ">&".fileno($newio) or croak "Can't redirect $fh - $!";
+    }
 
     bless [$$, $fh, $saved, $capture, $newio, $newio_file], $class;
 }
@@ -80,16 +109,23 @@ sub DESTROY {
     # restore the original filehandle
     my $fh_ref = Symbol::qualify_to_ref($fh);
     select((select ($fh_ref), $|=1)[0]);
-    open $fh_ref, ">&". fileno($saved) or croak "Can't restore $fh - $!";
+    if (defined $saved) {
+        open $fh_ref, ">&". fileno($saved) or croak "Can't restore $fh - $!";
+    }
+    else {
+        close $fh_ref;
+    }
 
-    # transfer captured data to the scalar reference
+    # transfer captured data to the scalar reference if we didn't merge
     my ($capture, $newio, $newio_file) = @{$self}[3..5];
-    seek $newio, 0, 0;
-    $$capture = do {local $/; <$newio>};
-    close $newio;
+    if ($newio_file) {
+        seek $newio, 0, 0;
+        $$capture = do {local $/; <$newio>};
+        close $newio;
+    }
 
     # Cleanup
-    return unless -e $newio_file;
+    return unless defined $newio_file && -e $newio_file;
     unlink $newio_file or carp "Couldn't remove temp file '$newio_file' - $!";
 }
 
@@ -111,34 +147,37 @@ This documentation describes version %%VERSION%%.
 
 = SYNOPSIS
 
-    use IO::CaptureOutput qw(capture capture_exec qxx);
+    use IO::CaptureOutput qw(capture capture_exec);
 
     my ($stdout, $stderr);
-    capture sub {noisy(@args)}, \$stdout, \$stderr;
+
     sub noisy {
-        my @args = @_;
         warn "this sub prints to stdout and stderr!";
-        ...
-        print "finished";
+        print "arguments: @_";
     }
 
+    capture sub {noisy(@args)}, \$stdout, \$stderr;
+
     ($stdout, $stderr) = capture_exec( 'perl', '-e', 
-        'print "Hello "; print STDERR "World!"');
+        'print "Hello"; print STDERR "World!"');
 
 = DESCRIPTION
 
-This module provides routines for capturing STDOUT and STDERR from forked
-system calls (e.g. {system()}, {fork()}) and from XS/C modules.
+This module provides routines for capturing STDOUT and STDERR from perl 
+subroutines, forked system calls (e.g. {system()}, {fork()}) and from 
+XS or C modules.
 
 = FUNCTIONS
 
-The following functions are be exported on demand.
+The following functions will be exported on demand.
 
-== {capture(\&subroutine, \$output, \$error)}
+== capture()
+
+    capture(\&subroutine, \$stdout, \$stderr);
 
 Captures everything printed to {STDOUT} and {STDERR} for the duration of
-{&subroutine}. {$output} and {$error} are optional scalar references that
-will contain {STDOUT} and {STDERR} respectively.
+{&subroutine}. {$stdout} and {$stderr} are optional scalars that will contain
+{STDOUT} and {STDERR} respectively. 
 
 Returns the return value(s) of {&subroutine}. The sub is called in the same
 context as {capture()} was called e.g.:
@@ -147,32 +186,57 @@ context as {capture()} was called e.g.:
     $rv = capture(sub {wantarray}); # returns defined, but not true
     capture(sub {wantarray});       # void, returns undef
 
-{capture()} is able to trap output from subprocesses and C code, which
-traditional {tie()} methods are unable to capture.
+{capture()} is able to capture output from subprocesses and C code, which
+traditional {tie()} methods of output capture are unable to do.
+
+If the two scalar references refer to the same scalar, then {STDERR} will be
+merged to {STDOUT} before capturing and the scalar will hold the combined
+output of both.
+
+    capture(\&subroutine, \$combined, \$combined);
 
 *Note:* {capture()} will only capture output that has been written or flushed
 to the filehandle.
 
-== {capture_exec(@args)}
+== capture_exec()
+
+    ($stdout, $stderr) = capture_exec(@args);
 
 Captures and returns the output from {system(@args)}. In scalar context,
 {capture_exec()} will return what was printed to {STDOUT}. In list context,
 it returns what was printed to {STDOUT} and {STDERR}
 
-    my $output = capture_exec('perl', '-e', 'print "hello world"');
+    $stdout = capture_exec('perl', '-e', 'print "hello world"');
 
-    my ($output, $error) = capture_exec('perl', '-e', 'warn "Test"');
+    ($stdout, $stderr) = capture_exec('perl', '-e', 'warn "Test"');
 
-{capture_exec} passes its arguments to {CORE::system} it can take advantage
-of the shell quoting, which makes it a handy and slightly more portable
-alternative to backticks, piped {open()} and {IPC::Open3}.
+{capture_exec} passes its arguments to {system()} and on MSWin32 will protect
+arguments with shell quotes if necessary.  This makes it a handy and slightly
+more portable alternative to backticks, piped {open()} and {IPC::Open3}.
 
 You can check the exit status of the {system()} call with the {$?}
 variable. See [perlvar] for more information.
 
-== {qxx(@args)}
+== capture_exec_combined()
 
-This is an alias of {capture_exec}
+    $combined = capture_exec_combined(
+        'perl', '-e', 'print "hello\n"', 'warn "Test\n"
+    );
+
+This is just like {capture_exec()}, except that it merges {STDERR} with {STDOUT}
+before capturing output and returns a single scalar.
+
+*Note:* there is no guarantee that text printed to {STDOUT} and {STDERR} in the
+subprocess will be appear in order. The actual order will depend on how IO
+buffering is handled in the subprocess.
+
+== qxx()
+
+This is an alias for {capture_exec()}.
+
+== qxy()
+
+This is an alias for {capture_exec_combined()}.
 
 = SEE ALSO
 
